#include <cstdlib>
#include <unistd.h> // for usleep
#include <math.h> // for math functions
#include "vibes.h"

using namespace std;

void draw_road(float road_radius);
void draw_robot(float x, float road_radius);
void f(float x, float v, float u, float& xdot, float& vdot);
void euler(float dt, float xdot, float vdot, float& x, float& v);

int main()
{
  const float road_radius = 100. / (2.*M_PI);
  const float dt = 0.05;
  float x = 0., v = 3., u = 1.;

  vibes::beginDrawing();
  vibes::newFigure("Road");
  vibes::setFigureProperties("Road", vibesParams("x", 200, "y", 200, "width", 400, "height", 400));
  vibes::axisLimits(-20., 20., -20., 20.);

  for(float t = 0. ; t < 100. ; t+=dt)
  {
    float xdot, vdot;

    f(v, u, xdot, vdot);
    euler(dt, xdot, vdot, x, v);

    draw_road(road_radius);
    draw_robot(x, road_radius);

    usleep(dt * 500000.); // animation speed
  }

  vibes::endDrawing();
  return EXIT_SUCCESS;
}

void draw_road(float road_radius)
{
  vibes::clearFigure("Road");
  vibes::drawCircle(0., 0., road_radius + 2.5, "#B8D3DA[#B8D3DA]", vibesParams("figure", "Road"));
  vibes::drawCircle(0., 0., road_radius - 2.5, "white[white]", vibesParams("figure", "Road"));
}

void draw_robot(float x, float road_radius)
{
  float theta = x / road_radius;
  vibes::drawTank(road_radius*cos(theta), road_radius*sin(theta),
                  (theta + M_PI/2.) * 180. / M_PI,
                  4., "black[white]", vibesParams("figure", "Road"));
}

void f(float x, float v, float u, float& xdot, float& vdot)
{
  xdot = v;
  vdot = u;
}

void euler(float dt, float xdot, float vdot, float& x, float& v)
{
  v += vdot * dt;
  x += xdot * dt;
}